@@ -1,5 +1,5 @@
 /*
- * "$Id: print-color.c,v 1.143 2011/03/08 13:03:05 rlk Exp $"
+ * "$Id: print-color.c,v 1.148 2014/01/24 01:55:56 rlk Exp $"
  *
  *   Gutenprint color management module - traditional Gutenprint algorithm.
  *
@@ -110,6 +110,38 @@ static const channel_param_t raw_channel_params[] =
   { 29, "GammaCh29", "CurveCh29", "GammaCh29", "CurveCh29" },
   { 30, "GammaCh30", "CurveCh30", "GammaCh30", "CurveCh30" },
   { 31, "GammaCh31", "CurveCh31", "GammaCh31", "CurveCh31" },
+  { 32, "GammaCh32", "CurveCh32", "GammaCh32", "CurveCh32" },
+  { 33, "GammaCh33", "CurveCh33", "GammaCh33", "CurveCh33" },
+  { 34, "GammaCh34", "CurveCh34", "GammaCh34", "CurveCh34" },
+  { 35, "GammaCh35", "CurveCh35", "GammaCh35", "CurveCh35" },
+  { 36, "GammaCh36", "CurveCh36", "GammaCh36", "CurveCh36" },
+  { 37, "GammaCh37", "CurveCh37", "GammaCh37", "CurveCh37" },
+  { 38, "GammaCh38", "CurveCh38", "GammaCh38", "CurveCh38" },
+  { 39, "GammaCh39", "CurveCh39", "GammaCh39", "CurveCh39" },
+  { 40, "GammaCh40", "CurveCh40", "GammaCh40", "CurveCh40" },
+  { 41, "GammaCh41", "CurveCh41", "GammaCh41", "CurveCh41" },
+  { 42, "GammaCh42", "CurveCh42", "GammaCh42", "CurveCh42" },
+  { 43, "GammaCh43", "CurveCh43", "GammaCh43", "CurveCh43" },
+  { 44, "GammaCh44", "CurveCh44", "GammaCh44", "CurveCh44" },
+  { 45, "GammaCh45", "CurveCh45", "GammaCh45", "CurveCh45" },
+  { 46, "GammaCh46", "CurveCh46", "GammaCh46", "CurveCh46" },
+  { 47, "GammaCh47", "CurveCh47", "GammaCh47", "CurveCh47" },
+  { 48, "GammaCh48", "CurveCh48", "GammaCh48", "CurveCh48" },
+  { 49, "GammaCh49", "CurveCh49", "GammaCh49", "CurveCh49" },
+  { 50, "GammaCh50", "CurveCh50", "GammaCh50", "CurveCh50" },
+  { 51, "GammaCh51", "CurveCh51", "GammaCh51", "CurveCh51" },
+  { 52, "GammaCh52", "CurveCh52", "GammaCh52", "CurveCh52" },
+  { 53, "GammaCh53", "CurveCh53", "GammaCh53", "CurveCh53" },
+  { 54, "GammaCh54", "CurveCh54", "GammaCh54", "CurveCh54" },
+  { 55, "GammaCh55", "CurveCh55", "GammaCh55", "CurveCh55" },
+  { 56, "GammaCh56", "CurveCh56", "GammaCh56", "CurveCh56" },
+  { 57, "GammaCh57", "CurveCh57", "GammaCh57", "CurveCh57" },
+  { 58, "GammaCh58", "CurveCh58", "GammaCh58", "CurveCh58" },
+  { 59, "GammaCh59", "CurveCh59", "GammaCh59", "CurveCh59" },
+  { 60, "GammaCh60", "CurveCh60", "GammaCh60", "CurveCh60" },
+  { 61, "GammaCh61", "CurveCh61", "GammaCh61", "CurveCh61" },
+  { 62, "GammaCh62", "CurveCh62", "GammaCh62", "CurveCh62" },
+  { 63, "GammaCh63", "CurveCh63", "GammaCh63", "CurveCh63" },
 };
 
 static const int raw_channel_param_count =
@@ -416,6 +448,38 @@ static const float_param_t float_parameters[] =
   RAW_GAMMA_CHANNEL(29),
   RAW_GAMMA_CHANNEL(30),
   RAW_GAMMA_CHANNEL(31),
+  RAW_GAMMA_CHANNEL(32),
+  RAW_GAMMA_CHANNEL(33),
+  RAW_GAMMA_CHANNEL(34),
+  RAW_GAMMA_CHANNEL(35),
+  RAW_GAMMA_CHANNEL(36),
+  RAW_GAMMA_CHANNEL(37),
+  RAW_GAMMA_CHANNEL(38),
+  RAW_GAMMA_CHANNEL(39),
+  RAW_GAMMA_CHANNEL(40),
+  RAW_GAMMA_CHANNEL(41),
+  RAW_GAMMA_CHANNEL(42),
+  RAW_GAMMA_CHANNEL(43),
+  RAW_GAMMA_CHANNEL(44),
+  RAW_GAMMA_CHANNEL(45),
+  RAW_GAMMA_CHANNEL(46),
+  RAW_GAMMA_CHANNEL(47),
+  RAW_GAMMA_CHANNEL(48),
+  RAW_GAMMA_CHANNEL(49),
+  RAW_GAMMA_CHANNEL(50),
+  RAW_GAMMA_CHANNEL(51),
+  RAW_GAMMA_CHANNEL(52),
+  RAW_GAMMA_CHANNEL(53),
+  RAW_GAMMA_CHANNEL(54),
+  RAW_GAMMA_CHANNEL(55),
+  RAW_GAMMA_CHANNEL(56),
+  RAW_GAMMA_CHANNEL(57),
+  RAW_GAMMA_CHANNEL(58),
+  RAW_GAMMA_CHANNEL(59),
+  RAW_GAMMA_CHANNEL(60),
+  RAW_GAMMA_CHANNEL(61),
+  RAW_GAMMA_CHANNEL(62),
+  RAW_GAMMA_CHANNEL(63),
   {
     {
       "LUTDumpFile", N_("LUT dump file"), N_("Advanced Output Control"),
@@ -589,6 +653,38 @@ static curve_param_t curve_parameters[] =
   RAW_CURVE_CHANNEL(29),
   RAW_CURVE_CHANNEL(30),
   RAW_CURVE_CHANNEL(31),
+  RAW_CURVE_CHANNEL(32),
+  RAW_CURVE_CHANNEL(33),
+  RAW_CURVE_CHANNEL(34),
+  RAW_CURVE_CHANNEL(35),
+  RAW_CURVE_CHANNEL(36),
+  RAW_CURVE_CHANNEL(37),
+  RAW_CURVE_CHANNEL(38),
+  RAW_CURVE_CHANNEL(39),
+  RAW_CURVE_CHANNEL(40),
+  RAW_CURVE_CHANNEL(41),
+  RAW_CURVE_CHANNEL(42),
+  RAW_CURVE_CHANNEL(43),
+  RAW_CURVE_CHANNEL(44),
+  RAW_CURVE_CHANNEL(45),
+  RAW_CURVE_CHANNEL(46),
+  RAW_CURVE_CHANNEL(47),
+  RAW_CURVE_CHANNEL(48),
+  RAW_CURVE_CHANNEL(49),
+  RAW_CURVE_CHANNEL(50),
+  RAW_CURVE_CHANNEL(51),
+  RAW_CURVE_CHANNEL(52),
+  RAW_CURVE_CHANNEL(53),
+  RAW_CURVE_CHANNEL(54),
+  RAW_CURVE_CHANNEL(55),
+  RAW_CURVE_CHANNEL(56),
+  RAW_CURVE_CHANNEL(57),
+  RAW_CURVE_CHANNEL(58),
+  RAW_CURVE_CHANNEL(59),
+  RAW_CURVE_CHANNEL(60),
+  RAW_CURVE_CHANNEL(61),
+  RAW_CURVE_CHANNEL(62),
+  RAW_CURVE_CHANNEL(63),
 };
 
 static const int curve_parameter_count =
@@ -831,7 +927,7 @@ compute_gcr_curve(const stp_vars_t *vars)
       }
   curve = stp_curve_create(STP_CURVE_WRAP_NONE);
   stp_curve_set_bounds(curve, 0, 65535);
-  STPI_ASSERT(stp_curve_set_data(curve, lut->steps, tmp_data), v);
+  STPI_ASSERT(stp_curve_set_data(curve, lut->steps, tmp_data), vars);
   stp_free(tmp_data);
   return curve;
 }
@@ -1575,7 +1671,7 @@ stpi_color_traditional_describe_parameter(const stp_vars_t *v,
 		description->is_active = 1;
 	      else
 		description->is_active = 0;
-	      if (param->color_only &&
+	      if (param->color_only && color_description &&
 		  !(color_description->channels & ~CMASK_K))
 		description->is_active = 0;
 	    }
@@ -1698,7 +1794,7 @@ stpi_color_traditional_describe_parameter(const stp_vars_t *v,
 		description->is_active = 1;
 	      else
 		description->is_active = 0;
-	      if (param->color_only &&
+	      if (param->color_only && color_description &&
 		  !(color_description->channels & ~CMASK_K))
 		description->is_active = 0;
 	    }
@@ -1734,7 +1830,7 @@ static const stp_colorfuncs_t stpi_color_traditional_colorfuncs =
   &stpi_color_traditional_describe_parameter
 };
 
-static const stp_color_t stpi_color_traditional_module_data =
+static stp_color_t stpi_color_traditional_module_data =
   {
     "traditional",
     N_("Traditional Gutenprint color conversion"),
