@@ -1,5 +1,5 @@
 /*
- * "$Id: print-papers.c,v 1.38 2008/01/15 03:48:46 rlk Exp $"
+ * "$Id: print-papers.c,v 1.43 2014/01/12 01:30:44 rlk Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -113,6 +113,8 @@ stpi_paper_create(stp_papersize_t *p)
 	(const stp_papersize_t *) stp_list_item_get_data(paper_item);
       if (ep && !strcmp(p->name, ep->name))
 	{
+	  stp_erprintf("Duplicate paper size `%s'\n",
+		       p->name);
 	  stpi_paper_freefunc(p);
 	  return 1;
 	}
@@ -125,30 +127,6 @@ stpi_paper_create(stp_papersize_t *p)
   return 0;
 }
 
-static int
-stpi_paper_destroy(stp_papersize_t *p)
-{
-  stp_list_item_t *paper_item;
-  check_paperlist();
-
-  /* Check if paper exists */
-  paper_item = stp_list_get_start(paper_list);
-  while (paper_item)
-    {
-      const stp_papersize_t *ep = (const stp_papersize_t *)
-	stp_list_item_get_data(paper_item);
-      if (ep && !strcmp(p->name, ep->name))
-	{
-	  stp_list_item_destroy (paper_list, paper_item);
-	  return 0;
-	}
-      paper_item = stp_list_item_next(paper_item);
-    }
-  /* Paper did not exist */
-  return 1;
-}
-
-
 int
 stp_known_papersizes(void)
 {
@@ -187,7 +165,7 @@ paper_size_mismatch(int l, int w, const stp_papersize_t *val)
 {
   int hdiff = abs(l - (int) val->height);
   int vdiff = abs(w - (int) val->width);
-  return hdiff + vdiff;
+  return hdiff > vdiff ? hdiff : vdiff;
 }
 
 const stp_papersize_t *
@@ -203,11 +181,17 @@ stp_get_papersize_by_size(int l, int w)
       val = stp_get_papersize_by_index(i);
 
       if (val->width == w && val->height == l)
-	return val;
+	{
+	  if (val->top == 0 && val->left == 0 &&
+	      val->bottom == 0 && val->right == 0)
+	    return val;
+	  else
+	    ref = val;
+	}
       else
 	{
 	  int myscore = paper_size_mismatch(l, w, val);
-	  if (myscore < score && myscore < 20)
+	  if (myscore < score && myscore < 5)
 	    {
 	      ref = val;
 	      score = myscore;
@@ -217,6 +201,29 @@ stp_get_papersize_by_size(int l, int w)
   return ref;
 }
 
+const stp_papersize_t *
+stp_get_papersize_by_size_exact(int l, int w)
+{
+  const stp_papersize_t *ref = NULL;
+  const stp_papersize_t *val = NULL;
+  int i;
+  int sizes = stp_known_papersizes();
+  for (i = 0; i < sizes; i++)
+    {
+      val = stp_get_papersize_by_index(i);
+
+      if (val->width == w && val->height == l)
+	{
+	  if (val->top == 0 && val->left == 0 &&
+	      val->bottom == 0 && val->right == 0)
+	    return val;
+	  else
+	    ref = val;
+	}
+    }
+  return ref;
+}
+
 void
 stp_default_media_size(const stp_vars_t *v,	/* I */
 		       int  *width,		/* O - Width in points */
@@ -279,10 +286,6 @@ stp_xml_process_paper(stp_mxml_node_t *paper) /* The paper node */
     name = 0,			/* Check name is present */
     height = 0,			/* Check height is present */
     width = 0,			/* Check width is present */
-    left = 0,			/* Check left is present */
-    right = 0,			/* Check right is present */
-    bottom = 0,			/* Check bottom is present */
-    top = 0,			/* Check top is present */
     unit = 0;			/* Check unit is present */
 
   if (stp_get_debug_level() & STP_DBG_XML)
@@ -341,25 +344,21 @@ stp_xml_process_paper(stp_mxml_node_t *paper) /* The paper node */
 	    {
 	      stmp = stp_mxmlElementGetAttr(prop, "value");
 	      outpaper->left = stp_xmlstrtoul(stmp);
-	      left = 1;
 	    }
 	  if (!strcmp(prop_name, "right"))
 	    {
 	      stmp = stp_mxmlElementGetAttr(prop, "value");
 	      outpaper->right = stp_xmlstrtoul(stmp);
-	      right = 1;
 	    }
 	  if (!strcmp(prop_name, "bottom"))
 	    {
 	      stmp = stp_mxmlElementGetAttr(prop, "value");
 	      outpaper->bottom = stp_xmlstrtoul(stmp);
-	      bottom = 1;
 	    }
 	  if (!strcmp(prop_name, "top"))
 	    {
 	      stmp = stp_mxmlElementGetAttr(prop, "value");
 	      outpaper->top = stp_xmlstrtoul(stmp);
-	      top = 1;
 	    }
 	  if (!strcmp(prop_name, "unit"))
 	    {
