@@ -1,5 +1,5 @@
 /*
- * "$Id: print-pcl.c,v 1.150 2008/04/26 01:15:11 rlk Exp $"
+ * "$Id: print-pcl.c,v 1.162 2014/01/04 23:07:22 rlk Exp $"
  *
  *   Print plug-in HP PCL driver for the GIMP.
  *
@@ -37,7 +37,6 @@
 #include <string.h>
 
 /* #define DEBUG */
-/* #define PCL_DEBUG_DISABLE_COMPRESSION */
 /* #define PCL_DEBUG_DISABLE_BLANKLINE_REMOVAL */
 
 /*
@@ -205,6 +204,7 @@ static const pcl_t pcl_media_types[] =
 
 #define PCL_PAPERSOURCE_STANDARD	0	/* Don't output code */
 #define PCL_PAPERSOURCE_MANUAL		2
+#define PCL_PAPERSOURCE_MANUAL_ADJ	(PCL_PAPERSOURCE_MANUAL + PAPERSOURCE_ADJ_GUIDE)
 #define PCL_PAPERSOURCE_ENVELOPE	3	/* Not used */
 
 /* LaserJet types */
@@ -236,6 +236,7 @@ static const pcl_t pcl_media_sources[] =
 {
     { "Standard", N_ ("Standard"), PCL_PAPERSOURCE_STANDARD},
     { "Manual", N_ ("Manual"), PCL_PAPERSOURCE_MANUAL},
+    { "ManualAdj", N_ ("Manual - Movable Guides"), PCL_PAPERSOURCE_MANUAL_ADJ},
 /*  {"Envelope", PCL_PAPERSOURCE_ENVELOPE}, */
     { "MultiPurposeAdj", N_ ("Tray 1 - Movable Guides"), PCL_PAPERSOURCE_LJ_TRAY1_ADJ},
     { "MultiPurpose", N_ ("Tray 1"), PCL_PAPERSOURCE_LJ_TRAY1},
@@ -274,26 +275,16 @@ static const pcl_t pcl_resolutions[] =
 };
 #define NUM_RESOLUTIONS		(sizeof(pcl_resolutions) / sizeof (pcl_t))
 
-static void
-pcl_describe_resolution(const stp_vars_t *v, int *x, int *y)
+static const pcl_t pcl_qualities[] =
 {
-  int i;
-  const char *resolution = stp_get_string_parameter(v, "Resolution");
-  if (resolution)
-    {
-      for (i = 0; i < NUM_RESOLUTIONS; i++)
-	{
-	  if (!strcmp(resolution, pcl_resolutions[i].pcl_name))
-	    {
-	      *x = pcl_resolutions[i].p0;
-	      *y = pcl_resolutions[i].p1;
-	      return;
-	    }
-	}
-    }
-  *x = -1;
-  *y = -1;
-}
+    { "Draft", N_("Draft"), PCL_RES_150_150, 150, 150},
+    { "Standard", N_("Standard"), PCL_RES_300_300, 300, 300},
+    { "High", N_("High"), PCL_RES_600_600, 600, 600},
+    { "High", N_("Standard"), PCL_RES_600_300, 600, 300},
+    { "Photo", N_("Photo"), PCL_RES_1200_600, 1200, 600},
+    { "Photo", N_("Photo"), PCL_RES_2400_600, 2400, 600},
+};
+#define NUM_QUALITIES		(sizeof(pcl_qualities) / sizeof (pcl_t))
 
 typedef struct {
   int top_margin;
@@ -575,6 +566,29 @@ static const short ljsmall_papersizes[] =
 };
 
 static const short ljbig_papersizes[] =
+{
+  PCL_PAPERSIZE_EXECUTIVE,
+  PCL_PAPERSIZE_STATEMENT,
+  PCL_PAPERSIZE_LETTER,
+  PCL_PAPERSIZE_LEGAL,
+  PCL_PAPERSIZE_TABLOID,
+  PCL_PAPERSIZE_SUPER_B,
+  PCL_PAPERSIZE_A5,
+  PCL_PAPERSIZE_A4,
+  PCL_PAPERSIZE_A3,
+  PCL_PAPERSIZE_JIS_B5,
+  PCL_PAPERSIZE_JIS_B4,                /* Guess */
+  PCL_PAPERSIZE_4x6,
+  PCL_PAPERSIZE_5x8,
+  PCL_PAPERSIZE_MONARCH_ENV,
+  PCL_PAPERSIZE_COMMERCIAL10_ENV,
+  PCL_PAPERSIZE_DL_ENV,
+  PCL_PAPERSIZE_C5_ENV,
+  PCL_PAPERSIZE_C6_ENV,
+  -1,
+};
+
+static const short ljtabloid_papersizes[] =
 {
   PCL_PAPERSIZE_EXECUTIVE,
   PCL_PAPERSIZE_STATEMENT,
@@ -621,6 +635,7 @@ static const short new_papertypes[] =
 static const short laserjet_papersources[] =
 {
   PCL_PAPERSOURCE_STANDARD,
+  PCL_PAPERSOURCE_MANUAL_ADJ,
   PCL_PAPERSOURCE_MANUAL,
   PCL_PAPERSOURCE_LJ_TRAY1_ADJ,
   PCL_PAPERSOURCE_LJ_TRAY1,
@@ -1062,6 +1077,19 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* PCL-4 with large paper, no expanded A4 margins */
+  { 24,
+    13 * 72, 19 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 18, 18},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ,
+    ljbig_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* LaserJet III series */
   { 3,
     17 * 72 / 2, 14 * 72,
@@ -1114,6 +1142,32 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* PCL-5 with large paper, no expanded margins */
+  { 34,
+    13 * 72, 19 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 18, 18},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE,
+    ljbig_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
+  /* PCL-5 with tabloid paper, no expanded margins */
+  { 35,
+    118 * 72 / 10, 17 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300,
+    {12, 12, 18, 18},
+    {12, 12, 18, 18},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE,
+    ljtabloid_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* LaserJet 4L */
   { 4,
     17 * 72 / 2, 14 * 72,
@@ -1196,6 +1250,34 @@ static const pcl_cap_t pcl_model_capabilities[] =
     emptylist,
     laserjet_papersources,
   },
+  /* PCL-5c/5e/6/XL with large paper, no expanded A4 margins */
+  { 63,
+    13 * 72, 19 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300 | PCL_RES_600_600,
+    {12, 12, 18, 18},
+    {12, 12, 18, 18},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_NEW_ERG | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE |
+      PCL_PRINTER_DUPLEX,
+    ljbig_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
+  /* PCL-5c/5e/6/XL with tabloid paper, no expanded A4 margins */
+  { 64,
+    118 * 72 / 10, 17 * 72,
+    1, 1,				/* Min paper size */
+    PCL_RES_150_150 | PCL_RES_300_300 | PCL_RES_600_600,
+    {12, 12, 18, 18},
+    {12, 12, 18, 18},	/* Check/Fix */
+    PCL_COLOR_NONE,
+    PCL_PRINTER_LJ | PCL_PRINTER_NEW_ERG | PCL_PRINTER_TIFF | PCL_PRINTER_BLANKLINE |
+      PCL_PRINTER_DUPLEX,
+    ljtabloid_papersizes,
+    emptylist,
+    laserjet_papersources,
+  },
   /* LaserJet 5Si */
   { 7,
     13 * 72, 19 * 72,
@@ -1261,52 +1343,58 @@ static const char standard_hue_adjustment[] =
 static const stp_parameter_t the_parameters[] =
 {
   {
-    "PageSize", N_("Page Size"), N_("Basic Printer Setup"),
+    "PageSize", N_("Page Size"), "Color=No,Category=Basic Printer Setup",
     N_("Size of the paper being printed to"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "MediaType", N_("Media Type"), N_("Basic Printer Setup"),
+    "MediaType", N_("Media Type"), "Color=Yes,Category=Basic Printer Setup",
     N_("Type of media (plain paper, photo paper, etc.)"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "InputSlot", N_("Media Source"), N_("Basic Printer Setup"),
+    "InputSlot", N_("Media Source"), "Color=No,Category=Basic Printer Setup",
     N_("Source (input slot) of the media"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "Resolution", N_("Resolution"), N_("Basic Printer Setup"),
-    N_("Resolution and quality of the print"),
+    "Quality", N_("Print Quality"), "Color=Yes,Category=Basic Output Adjustment",
+    N_("Print Quality"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 0, 0
   },
   {
-    "InkType", N_("Ink Type"), N_("Advanced Printer Setup"),
+    "Resolution", N_("Resolution"), "Color=Yes,Category=Basic Printer Setup",
+    N_("Resolution of the print"),
+    STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
+  },
+  {
+    "InkType", N_("Ink Type"), "Color=Yes,Category=Advanced Printer Setup",
     N_("Type of ink in the printer"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "InkChannels", N_("Ink Channels"), N_("Advanced Printer Functionality"),
+    "InkChannels", N_("Ink Channels"), "Color=Yes,Category=Advanced Printer Functionality",
     N_("Ink Channels"),
     STP_PARAMETER_TYPE_INT, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_INTERNAL, 0, 0, -1, 0, 0
+    STP_PARAMETER_LEVEL_INTERNAL, 0, 0, STP_CHANNEL_NONE, 0, 0
   },
   {
-    "PrintingMode", N_("Printing Mode"), N_("Core Parameter"),
+    "PrintingMode", N_("Printing Mode"), "Color=Yes,Category=Core Parameter",
     N_("Printing Output Mode"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "Duplex", N_("Double-Sided Printing"), N_("Basic Printer Setup"),
+    "Duplex", N_("Double-Sided Printing"), "Color=No,Category=Basic Printer Setup",
     N_("Duplex/Tumble Setting"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -1326,7 +1414,7 @@ static const float_param_t float_parameters[] =
 {
   {
     {
-      "CyanDensity", N_("Cyan Density"), N_("Output Level Adjustment"),
+      "CyanDensity", N_("Cyan Density"), "Color=Yes,Category=Output Level Adjustment",
       N_("Adjust the cyan density"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
       STP_PARAMETER_LEVEL_ADVANCED, 0, 1, 1, 1, 0
@@ -1334,7 +1422,7 @@ static const float_param_t float_parameters[] =
   },
   {
     {
-      "MagentaDensity", N_("Magenta Density"), N_("Output Level Adjustment"),
+      "MagentaDensity", N_("Magenta Density"), "Color=Yes,Category=Output Level Adjustment",
       N_("Adjust the magenta density"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
       STP_PARAMETER_LEVEL_ADVANCED, 0, 1, 2, 1, 0
@@ -1342,7 +1430,7 @@ static const float_param_t float_parameters[] =
   },
   {
     {
-      "YellowDensity", N_("Yellow Density"), N_("Output Level Adjustment"),
+      "YellowDensity", N_("Yellow Density"), "Color=Yes,Category=Output Level Adjustment",
       N_("Adjust the yellow density"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
       STP_PARAMETER_LEVEL_ADVANCED, 0, 1, 3, 1, 0
@@ -1350,7 +1438,7 @@ static const float_param_t float_parameters[] =
   },
   {
     {
-      "BlackDensity", N_("Black Density"), N_("Output Level Adjustment"),
+      "BlackDensity", N_("Black Density"), "Color=Yes,Category=Output Level Adjustment",
       N_("Adjust the black density"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
       STP_PARAMETER_LEVEL_ADVANCED, 0, 1, 0, 1, 0
@@ -1358,18 +1446,18 @@ static const float_param_t float_parameters[] =
   },
   {
     {
-      "LightCyanTransition", N_("Light Cyan Transition"), N_("Advanced Ink Adjustment"),
+      "LightCyanTrans", N_("Light Cyan Transition"), "Color=Yes,Category=Advanced Ink Adjustment",
       N_("Light Cyan Transition"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
-      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, -1, 1, 0
+      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, STP_CHANNEL_NONE, 1, 0
     }, 0.0, 5.0, 1.0, 1
   },
   {
     {
-      "LightMagentaTransition", N_("Light Magenta Transition"), N_("Advanced Ink Adjustment"),
+      "LightMagentaTrans", N_("Light Magenta Transition"), "Color=Yes,Category=Advanced Ink Adjustment",
       N_("Light Magenta Transition"),
       STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
-      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, -1, 1, 0
+      STP_PARAMETER_LEVEL_ADVANCED4, 0, 1, STP_CHANNEL_NONE, 1, 0
     }, 0.0, 5.0, 1.0, 1
   },
 };
@@ -1491,6 +1579,51 @@ pcl_get_model_capabilities(int model)	/* I: Model */
   return &(pcl_model_capabilities[0]);
 }
 
+/*
+ * Determine the current resolution based on quality and resolution settings
+ */
+
+static void
+pcl_describe_resolution(const stp_vars_t *v, int *x, int *y)
+{
+  int i;
+  int model = stp_get_model_id(v);
+  const char *resolution = stp_get_string_parameter(v, "Resolution");
+  const char *quality;
+  const pcl_cap_t *caps = NULL;
+  if (resolution)
+    {
+      for (i = 0; i < NUM_RESOLUTIONS; i++)
+	{
+	  if (!strcmp(resolution, pcl_resolutions[i].pcl_name))
+	    {
+	      *x = pcl_resolutions[i].p0;
+	      *y = pcl_resolutions[i].p1;
+	      return;
+	    }
+	}
+    }
+  quality = stp_get_string_parameter(v, "Quality");
+  caps = pcl_get_model_capabilities(model);
+  if (quality && strcmp(quality, "None") == 0)
+    quality = "Standard";
+  if (quality)
+    {
+      for (i = 0; i < NUM_QUALITIES; i++)
+	{
+	  if ((caps->resolutions & pcl_qualities[i].pcl_code) &&
+	      !strcmp(quality, pcl_qualities[i].pcl_name))
+	    {
+	      *x = pcl_qualities[i].p0;
+	      *y = pcl_qualities[i].p1;
+	      return;
+	    }
+	}
+    }
+  *x = -1;
+  *y = -1;
+}
+
 /*
  * Convert Media size name into PCL media code for printer
  */
@@ -1591,7 +1724,7 @@ pcl_papersize_valid(const stp_papersize_t *pt,
     return(0);
 
 /*
- * Is it a recognised supported name?
+ * Is it a recognized supported name?
  */
 
   if (pcl_convert_media_size(pt->name, model) != -1)
@@ -1662,6 +1795,14 @@ pcl_parameters(const stp_vars_t *v, const char *name,
   stp_deprintf(STP_DBG_PCL, "Resolutions: %d\n", caps->resolutions);
   stp_deprintf(STP_DBG_PCL, "ColorType = %d, PrinterType = %d\n", caps->color_type, caps->stp_printer_type);
 
+  for (i = 0; i < the_parameter_count; i++)
+    if (strcmp(name, the_parameters[i].name) == 0)
+      {
+	stp_fill_parameter_settings(description, &(the_parameters[i]));
+	break;
+      }
+  description->deflt.str = NULL;
+
   for (i = 0; i < float_parameter_count; i++)
     if (strcmp(name, float_parameters[i].param.name) == 0)
       {
@@ -1670,15 +1811,8 @@ pcl_parameters(const stp_vars_t *v, const char *name,
 	description->deflt.dbl = float_parameters[i].defval;
 	description->bounds.dbl.upper = float_parameters[i].max;
 	description->bounds.dbl.lower = float_parameters[i].min;
-      }
-
-  for (i = 0; i < the_parameter_count; i++)
-    if (strcmp(name, the_parameters[i].name) == 0)
-      {
-	stp_fill_parameter_settings(description, &(the_parameters[i]));
 	break;
       }
-  description->deflt.str = NULL;
 
   if (strcmp(name, "PageSize") == 0)
     {
@@ -1735,15 +1869,11 @@ pcl_parameters(const stp_vars_t *v, const char *name,
   else if (strcmp(name, "Resolution") == 0)
   {
     description->bounds.str = stp_string_list_create();
-    description->deflt.str = NULL;
+    stp_string_list_add_string(description->bounds.str, "None", _("Default"));
+    description->deflt.str = "None";
     for (i = 0; i < NUM_RESOLUTIONS; i++)
       if (caps->resolutions & pcl_resolutions[i].pcl_code)
 	{
-	  if (pcl_resolutions[i].pcl_code >= PCL_RES_300_300 &&
-	      description->deflt.str == NULL)
-	    description->deflt.str =
-	      pcl_val_to_string(pcl_resolutions[i].pcl_code,
-				pcl_resolutions, NUM_RESOLUTIONS);
 	  stp_string_list_add_string
 	    (description->bounds.str,
 	     pcl_val_to_string(pcl_resolutions[i].pcl_code,
@@ -1751,8 +1881,31 @@ pcl_parameters(const stp_vars_t *v, const char *name,
 	     pcl_val_to_text(pcl_resolutions[i].pcl_code,
 			     pcl_resolutions, NUM_RESOLUTIONS));
 	}
-    if (description->deflt.str == NULL)
-      stp_erprintf("No default resolution set!\n");
+  }
+  else if (strcmp(name, "Quality") == 0)
+  {
+    int has_standard_quality = 0;
+    description->bounds.str = stp_string_list_create();
+    stp_string_list_add_string(description->bounds.str, "None",
+			       _("Manual Control"));
+    for (i = 0; i < NUM_QUALITIES; i++)
+      if (caps->resolutions & pcl_qualities[i].pcl_code)
+	{
+	  const char *qual =
+	    pcl_val_to_string(pcl_qualities[i].pcl_code,
+			      pcl_qualities, NUM_QUALITIES);
+	  if (! stp_string_list_is_present(description->bounds.str, qual))
+	    stp_string_list_add_string
+	      (description->bounds.str, qual,
+	       pcl_val_to_text(pcl_qualities[i].pcl_code,
+			       pcl_qualities, NUM_QUALITIES));
+	  if (strcmp(qual, "Standard") == 0)
+	    has_standard_quality = 1;
+	}
+    if (has_standard_quality)
+      description->deflt.str = "Standard";
+    else
+      description->deflt.str = "None";
   }
   else if (strcmp(name, "InkType") == 0)
   {
@@ -1815,8 +1968,8 @@ pcl_parameters(const stp_vars_t *v, const char *name,
       else
 	description->is_active = 0;
     }
-  else if (strcmp(name, "LightCyanTransition") == 0 ||
-	   strcmp(name, "LightMagentaTransition") == 0)
+  else if (strcmp(name, "LightCyanTrans") == 0 ||
+	   strcmp(name, "LightMagentaTrans") == 0)
     {
       if (caps->color_type & PCL_COLOR_CMYKcm &&
 	  stp_check_string_parameter(v, "PrintingMode", STP_PARAMETER_DEFAULTED) &&
@@ -2100,11 +2253,9 @@ get_double_param(stp_vars_t *v, const char *param)
 static int
 pcl_do_print(stp_vars_t *v, stp_image_t *image)
 {
-  int i;
   pcl_privdata_t privdata;
   int		status = 1;
   int		model = stp_get_model_id(v);
-  const char	*resolution = stp_get_string_parameter(v, "Resolution");
   const char	*media_size = stp_get_string_parameter(v, "PageSize");
   const char	*media_type = stp_get_string_parameter(v, "MediaType");
   const char	*media_source = stp_get_string_parameter(v, "InputSlot");
@@ -2123,23 +2274,19 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
 		*yellow,	/* Yellow bitmap data */
 		*lcyan,		/* Light Cyan bitmap data */
 		*lmagenta;	/* Light Magenta bitmap data */
-  int		page_width,	/* Width of page */
-		page_height,	/* Height of page */
-		page_left,
+  int		page_left,
 		page_top,
 		page_right,
 		page_bottom,
 		out_width,	/* Width of image on page */
 		out_height,	/* Height of image on page */
-		out_channels,	/* Output bytes per pixel */
 		errdiv,		/* Error dividend */
 		errmod,		/* Error modulus */
 		errval,		/* Current error value */
 		errline,	/* Current raster line */
 		errlast;	/* Last raster line loaded */
   unsigned	zero_mask;
-  int           image_height,
-                image_width;
+  int           image_height;
   const pcl_cap_t *caps;		/* Printer capabilities */
   int		planes = 3;	/* # of output planes */
   int		pcl_media_size; /* PCL media size code */
@@ -2151,7 +2298,7 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
   int		extra_left_margin = 0;
   stp_curve_t   *lum_adjustment;
   stp_curve_t   *hue_adjustment;
-  double density;
+  double        density;
 
   if (!stp_verify(v))
     {
@@ -2169,30 +2316,19 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
 
   stp_image_init(image);
   image_height = stp_image_height(image);
-  image_width = stp_image_width(image);
 
  /*
   * Figure out the output resolution...
   */
 
-  xdpi = 0;
-  ydpi = 0;
-  if (resolution)
-    {
-      for (i = 0; i < NUM_RESOLUTIONS; i++)
-	{
-	  if (!strcmp(resolution, pcl_resolutions[i].pcl_name))
-	    {
-	      xdpi = pcl_resolutions[i].p0;
-	      ydpi = pcl_resolutions[i].p1;
-	      break;
-	    }
-	}
-    }
+  pcl_describe_resolution(v, &xdpi, &ydpi);
 
   stp_deprintf(STP_DBG_PCL,"pcl: resolution=%dx%d\n",xdpi,ydpi);
-  if (xdpi == 0 || ydpi == 0)
-    return 0;
+  if (xdpi <= 0 || ydpi <= 0)
+    {
+      stp_eprintf(v, "No resolution found; cannot print.\n");
+      return 0;
+    }
 
  /*
   * Choose the correct color conversion function...
@@ -2238,11 +2374,8 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
 			  &page_bottom, &page_top);
   left -= page_left;
   top -= page_top;
-  page_width = page_right - page_left;
-  page_height = page_bottom - page_top;
 
   image_height = stp_image_height(image);
-  image_width = stp_image_width(image);
 
  /*
   * Set media size here because it is needed by the margin calculation code.
@@ -2498,13 +2631,12 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
     }
   }
 
-#ifndef PCL_DEBUG_DISABLE_COMPRESSION
-  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF)
+  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF &&
+      !(stp_get_debug_level() & STP_DBG_NO_COMPRESSION))
   {
     stp_puts("\033*b2M", v);			/* Mode 2 (TIFF) */
   }
   else
-#endif
   {
     stp_puts("\033*b0M", v);			/* Mode 0 (no compression) */
   }
@@ -2615,14 +2747,13 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
 
 /* Allocate buffer for pcl_mode2 tiff compression */
 
-#ifndef PCL_DEBUG_DISABLE_COMPRESSION
-  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF)
+  if ((caps->stp_printer_type & PCL_PRINTER_TIFF) == PCL_PRINTER_TIFF &&
+      !(stp_get_debug_level() & STP_DBG_NO_COMPRESSION))
   {
     privdata.comp_buf = stp_malloc((privdata.height + 128 + 7) * 129 / 128);
     privdata.writefunc = pcl_mode2;
   }
   else
-#endif
   {
     privdata.comp_buf = NULL;
     privdata.writefunc = pcl_mode0;
@@ -2674,7 +2805,7 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
       if (black && !privdata.do_cretb)
         stp_dither_set_inks_simple(v, STP_ECOLOR_K, 3, dot_sizes_use, 1.0, 1.0);
 
-      /* Note: no printer I know of does both CRet (4-level) and 6 colour, but
+      /* Note: no printer I know of does both CRet (4-level) and 6 color, but
 	 what the heck. variable_dither_ranges copied from print-escp2.c */
 
       if (privdata.do_6color)			/* Photo for 69x */
@@ -2696,7 +2827,7 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
     }
   else if (privdata.do_6color)
     {
-      /* Set light inks for 6 colour printers.
+      /* Set light inks for 6 color printers.
 	 Numbers copied from print-escp2.c */
       stp_dither_set_inks_full(v, STP_ECOLOR_C, 2, photo_dither_shades, 1.0,
 				0.31 / .5);
@@ -2706,29 +2837,29 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
   if (black)
     stp_channel_set_density_adjustment(v, STP_ECOLOR_K, 0,
 				       get_double_param(v, "BlackDensity") *
-				       get_double_param(v, "Density"));
+				       density);
   if (cyan)
     stp_channel_set_density_adjustment(v, STP_ECOLOR_C, 0,
 				       get_double_param(v, "CyanDensity") *
-				       get_double_param(v, "Density"));
+				       density);
   if (magenta)
     stp_channel_set_density_adjustment(v, STP_ECOLOR_M, 0,
 				       get_double_param(v, "MagentaDensity") *
-				       get_double_param(v, "Density"));
+				       density);
   if (yellow)
     stp_channel_set_density_adjustment(v, STP_ECOLOR_Y, 0,
 					get_double_param(v, "YellowDensity") *
-					get_double_param(v, "Density"));
+					density);
   if (lcyan)
     stp_channel_set_density_adjustment
       (v, STP_ECOLOR_C, 1, (get_double_param(v, "CyanDensity") *
-			get_double_param(v, "LightCyanTransition") *
-			get_double_param(v, "Density")));
+			get_double_param(v, "LightCyanTrans") *
+			density));
   if (lmagenta)
     stp_channel_set_density_adjustment
       (v, STP_ECOLOR_M, 1, (get_double_param(v, "MagentaDensity") *
-			get_double_param(v, "LightMagentaTransition") *
-			get_double_param(v, "Density")));
+			get_double_param(v, "LightMagentaTrans") *
+			density));
 
 
   if (!stp_check_curve_parameter(v, "HueMap", STP_PARAMETER_ACTIVE))
@@ -2743,7 +2874,7 @@ pcl_do_print(stp_vars_t *v, stp_image_t *image)
       stp_curve_destroy(lum_adjustment);
     }
 
-  out_channels = stp_color_init(v, image, 65536);
+  (void) stp_color_init(v, image, 65536);
 
   errdiv  = image_height / out_height;
   errmod  = image_height % out_height;
@@ -2943,4 +3074,3 @@ stp_module_t stp_module_data =
     print_pcl_module_exit,
     (void *) &print_pcl_module_data
   };
-
