@@ -1,7 +1,7 @@
 /*
- * "$Id: testpattern.c,v 1.54 2008/07/16 23:59:23 rlk Exp $"
+ * "$Id: testpattern.c,v 1.64 2014/05/17 21:56:46 rlk Exp $"
  *
- *   Test pattern generator for Gimp-Print
+ *   Test pattern generator for Gutenprint
  *
  *   Copyright 2001 Robert Krawitz <rlk@alum.mit.edu>
  *
@@ -22,7 +22,7 @@
 
 /*
  * This sample program may be used to generate test patterns.  It also
- * serves as an example of how to use the gimp-print API.
+ * serves as an example of how to use the Gutenprint API.
  *
  * As the purpose of this program is to allow fine grained control over
  * the output, it uses the raw CMYK output type.  This feeds 16 bits each
@@ -42,6 +42,8 @@
 #include <gutenprint/gutenprint-intl.h>
 #include <errno.h>
 
+#pragma GCC diagnostic ignored "-Wredundant-decls"
+
 extern int yyparse(void);
 
 static const char *Image_get_appname(stp_image_t *image);
@@ -71,6 +73,7 @@ double global_levels[STP_CHANNEL_LIMIT];
 double global_gammas[STP_CHANNEL_LIMIT];
 double global_gamma;
 int global_steps;
+int global_halt_on_error = 0;
 double global_ink_limit;
 int global_noblackline;
 int global_printer_width;
@@ -81,6 +84,7 @@ char *global_printer = NULL;
 double global_density;
 double global_xtop;
 double global_xleft;
+size_mode_t global_size_mode;
 double global_hsize;
 double global_vsize;
 const char *global_image_type;
@@ -92,11 +96,16 @@ int global_did_something;
 int global_noscale = 0;
 int global_suppress_output = 0;
 int global_quiet = 0;
+int global_fail_verify_ok = 0;
 char *global_output = NULL;
 FILE *output = NULL;
 int write_to_process = 0;
 int start_job = 0;
 int end_job = 0;
+int passes = 0;
+int failures = 0;
+int skipped = 0;
+size_t bytes_written = 0;
 
 static testpattern_t *static_testpatterns;
 
@@ -181,9 +190,15 @@ static void
 writefunc(void *file, const char *buf, size_t bytes)
 {
   FILE *prn = (FILE *)file;
+  bytes_written += bytes;
   if (! file)
     return;
-  if (!global_suppress_output || (file == stderr))
+  if (file == stderr)
+    {
+      if (!global_quiet)
+	fwrite(buf, 1, bytes, prn);
+    }
+  else if (!global_suppress_output)
     {
       fwrite(buf, 1, bytes, prn);
     }
@@ -249,6 +264,7 @@ initialize_global_parameters(void)
 static int
 do_print(void)
 {
+  int status = 0;
   stp_vars_t *v;
   const stp_printer_t *the_printer;
   int left, right, top, bottom;
@@ -291,6 +307,7 @@ do_print(void)
 	}
       return 2;
     }
+  bytes_written = 0;
   if (global_output)
     {
       if (strcmp(global_output, "-") == 0)
@@ -401,42 +418,112 @@ do_print(void)
 
   width = right - left;
   height = bottom - top;
-  top += height * global_xtop;
-  left += width * global_xleft;
-  if (global_steps > width)
-    global_steps = width;
 
+  switch (global_size_mode)
+    {
+    case SIZE_PT:
+      top += (int) (global_xtop + .5);
+      left += (int) (global_xleft + .5);
+      width = (int) (global_hsize + .5);
+      height = (int) (global_vsize + .5);
+      break;
+    case SIZE_IN:
+      top += (int) ((global_xtop * 72) + .5);
+      left += (int) ((global_xleft * 72) + .5);
+      width = (int) ((global_hsize * 72) + .5);
+      height = (int) ((global_vsize * 72) + .5);
+      break;
+    case SIZE_MM:
+      top += (int) ((global_xtop * 72 / 25.4) + .5);
+      left += (int) ((global_xleft * 72 / 25.4) + .5);
+      width = (int) ((global_hsize * 72 / 25.4) + .5);
+      height = (int) ((global_vsize * 72 / 25.4) + .5);
+      break;
+    case SIZE_RELATIVE:
+    default:
+      top += height * global_xtop;
+      left += width * global_xleft;
+      width *= global_hsize;
+      height *= global_vsize;
+      break;
+    }
+  stp_set_width(v, width);
+  stp_set_height(v, height);
 #if 0
   width = (width / global_steps) * global_steps;
   height = (height / global_n_testpatterns) * global_n_testpatterns;
 #endif
-  stp_set_width(v, width * global_hsize);
-  stp_set_height(v, height * global_vsize);
+  if (global_steps > width)
+    global_steps = width;
 
   global_printer_width = width * x / 72;
   global_printer_height = height * y / 72;
 
   global_band_height = global_printer_height / global_n_testpatterns;
+  if (global_band_height == 0)
+    global_band_height = 1;
   stp_set_left(v, left);
   stp_set_top(v, top);
 
   stp_merge_printvars(v, stp_printer_get_defaults(the_printer));
-  if (start_job)
+  if (stp_verify(v))
     {
-      stp_start_job(v, &theImage);
-      start_job = 0;
+      bytes_written = 0;
+      if (start_job)
+	{
+	  stp_start_job(v, &theImage);
+	  start_job = 0;
+	}
+      if (stp_print(v, &theImage) != 1)
+	{
+	  if (!global_quiet)
+	    fputs("FAILED", stderr);
+	  failures++;
+	  status = 2;
+	  if (global_halt_on_error)
+	    return status;
+	}
+      else if (bytes_written == 0)
+	{
+	  if (!global_quiet)
+	    fputs("FAILED: No output", stderr);
+	  failures++;
+	  status = 2;
+	  if (global_halt_on_error)
+	    return status;
+	}
+      else
+	passes++;
+      if (end_job)
+	{
+	  stp_end_job(v, &theImage);
+	  end_job = 0;
+	}
     }
-  if (stp_print(v, &theImage) != 1)
-    return 2;
-  if (end_job)
+  else
     {
-      stp_end_job(v, &theImage);
-      end_job = 0;
+      if (! global_fail_verify_ok)
+	{
+	  if (!global_quiet)
+	    fputs("FAILED", stderr);
+	  failures++;
+	  status = 2;
+	  if (global_halt_on_error)
+	    return status;
+	}
+      else
+	{
+	  if (!global_quiet)
+	    fputs("(skipped)", stderr);
+	  skipped++;
+	}
     }
+  if (!global_quiet)
+    fputc('\n', stderr);
   stp_vars_destroy(v);
   stp_free(static_testpatterns);
   static_testpatterns = NULL;
-  return 0;
+  return status;
 }
 
 int
@@ -447,7 +534,7 @@ main(int argc, char **argv)
   int global_status = 0;
   while (1)
     {
-      c = getopt(argc, argv, "nq");
+      c = getopt(argc, argv, "nqyH");
       if (c == -1)
 	break;
       switch (c)
@@ -458,6 +545,12 @@ main(int argc, char **argv)
 	case 'q':
 	  global_quiet = 1;
 	  break;
+	case 'y':
+	  global_fail_verify_ok = 1;
+	  break;
+	case 'H':
+	  global_halt_on_error = 1;
+	  break;
 	default:
 	  break;
 	}
@@ -474,6 +567,8 @@ main(int argc, char **argv)
 	global_status = 1;
     }
   close_output();
+  if (passes + failures + skipped > 1)
+    fprintf(stderr, "%d pass, %d fail, %d skipped\n", passes, failures, skipped);
   return global_status;
 }
 
@@ -706,6 +801,8 @@ fill_grid_##bits(unsigned char *data, size_t len, size_t scount,	\
 	  errlast = errline;						\
 	  s_data[0] = multiplier;					\
 	}								\
+      else								\
+	s_data[0] = 0;							\
       errval += errmod;							\
       errline += errdiv;						\
       if (errval >= xlen - 1)						\
@@ -1022,7 +1119,7 @@ Image_get_row(stp_image_t *image, unsigned char *data,
   int depth = global_channel_depth;
   if (! Image_is_valid)
     {
-      fprintf(stderr, "Calling Image_get_row with invalid image!\n");
+      fputs("Calling Image_get_row with invalid image!\n", stderr);
       abort();
     }
   if (static_testpatterns[0].type == E_IMAGE)
@@ -1032,56 +1129,64 @@ Image_get_row(stp_image_t *image, unsigned char *data,
 			     yyin);
       if (total_read != t->d.image.x * depth * global_bit_depth / 8)
 	{
-	  fprintf(stderr, "Read failed!\n");
+	  fputs("Read failed!\n", stderr);
 	  return STP_IMAGE_STATUS_ABORT;
 	}
       if (!global_quiet)
-	fprintf(stderr, ".");
+	fputc('.', stderr);
     }
   else
     {
       static int previous_band = -1;
+      static int printed_blackline = 0;
       int band = row / global_band_height;
-      if (previous_band == -1)
-	{
-	  fill_pattern(&(static_testpatterns[band]), data,
-		       global_printer_width, global_steps, depth,
-		       global_bit_depth / 8);
-	  previous_band = band;
-	  if (!global_quiet)
-	    fprintf(stderr, ".");
-	}
-      else if (row == global_printer_height - 1)
+      if (row == global_printer_height - 1 && ! global_noblackline)
 	fill_black(data, global_printer_width, global_steps,
 		   global_bit_depth / 8);
       else if (band >= global_n_testpatterns)
 	fill_white(data, global_printer_width, global_steps,
 		   global_bit_depth / 8);
-      else if (band != previous_band && band >= 0)
+      else
 	{
+	  if (band != previous_band)
+	    {
+	      if (! global_noblackline && printed_blackline == 0)
+		{
+		  fill_black(data, global_printer_width, global_steps,
+			     global_bit_depth / 8);
+		  printed_blackline = 1;
+		  return STP_IMAGE_STATUS_OK;
+		}
+	      else
+		{
+		  previous_band = band;
+		  printed_blackline = 0;
+		  if (! global_quiet)
+		    fputc('.', stderr);
+		}
+	    }
 	  fill_pattern(&(static_testpatterns[band]), data,
 		       global_printer_width, global_steps, depth,
 		       global_bit_depth / 8);
-	  previous_band = band;
-	  if (!global_quiet)
-	    fprintf(stderr, ".");
 	}
-      else
-	fill_pattern(&(static_testpatterns[band]), data,
-		     global_printer_width, global_steps, depth,
-		     global_bit_depth / 8);
     }
   return STP_IMAGE_STATUS_OK;
 }
 
+static void
+check_valid_image(const char *s)
+{
+  if (! Image_is_valid)
+    {
+      fputs(s, stderr);
+      abort();
+    }
+}
+
 static int
 Image_width(stp_image_t *image)
 {
-  if (! Image_is_valid)
-    {
-      fprintf(stderr, "Calling Image_width with invalid image!\n");
-      abort();
-    }
+  check_valid_image("Calling Image_width with invalid image!\n");
   if (static_testpatterns[0].type == E_IMAGE)
     return static_testpatterns[0].d.image.x;
   else
@@ -1091,11 +1196,7 @@ Image_width(stp_image_t *image)
 static int
 Image_height(stp_image_t *image)
 {
-  if (! Image_is_valid)
-    {
-      fprintf(stderr, "Calling Image_height with invalid image!\n");
-      abort();
-    }
+  check_valid_image("Calling Image_height with invalid image!\n");
   if (static_testpatterns[0].type == E_IMAGE)
     return static_testpatterns[0].d.image.y;
   else
@@ -1107,7 +1208,7 @@ Image_init(stp_image_t *image)
 {
   if (Image_is_valid)
     {
-      fprintf(stderr, "Calling Image_init with already valid image!\n");
+      fputs("Calling Image_init with already valid image!\n", stderr);
       abort();
     }
   Image_is_valid = 1;
@@ -1117,34 +1218,20 @@ Image_init(stp_image_t *image)
 static void
 Image_reset(stp_image_t *image)
 {
-  if (!Image_is_valid)
-    {
-      fprintf(stderr, "Calling Image_reset with invalid image!\n");
-      abort();
-    }
+  check_valid_image("Calling Image_reset with invalid image!\n");
  /* dummy function */
 }
 
 static void
 Image_conclude(stp_image_t *image)
 {
-  if (! Image_is_valid)
-    {
-      fprintf(stderr, "Calling Image_conclude with invalid image!\n");
-      abort();
-    }
+  check_valid_image("Calling Image_conclude with invalid image!\n");
   Image_is_valid = 0;
-  if (!global_quiet)
-    fprintf(stderr, "\n");
 }
 
 static const char *
 Image_get_appname(stp_image_t *image)
 {
-  if (! Image_is_valid)
-    {
-      fprintf(stderr, "Calling Image_get_appname with invalid image!\n");
-      abort();
-    }
+  check_valid_image("Calling Image_get_appname with invalid image!\n");
   return "Test Pattern";
 }
